package com.sun.rowset.internal;

import java.io.ObjectInputStream;
import java.sql.Savepoint;
import java.sql.DatabaseMetaData;
import java.util.Map;
import java.sql.ResultSet;
import javax.sql.rowset.serial.SQLInputImpl;
import sun.reflect.misc.ReflectUtil;
import javax.sql.rowset.serial.SerialStruct;
import javax.sql.rowset.serial.SerialBlob;
import javax.sql.rowset.serial.SerialClob;
import javax.sql.rowset.serial.SerialArray;
import java.sql.Array;
import java.sql.Clob;
import java.sql.Blob;
import java.sql.SQLData;
import java.sql.Struct;
import java.util.Vector;
import javax.sql.rowset.CachedRowSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import javax.sql.rowset.spi.SyncProviderException;
import javax.sql.rowset.RowSetMetaDataImpl;
import javax.sql.RowSetInternal;
import java.io.IOException;
import com.sun.rowset.JdbcRowSetResourceBundle;
import java.util.ArrayList;
import com.sun.rowset.CachedRowSetImpl;
import java.sql.ResultSetMetaData;
import java.sql.Connection;
import java.io.Serializable;
import javax.sql.rowset.spi.TransactionalWriter;

public class CachedRowSetWriter implements TransactionalWriter, Serializable
{
    private transient Connection con;
    private String selectCmd;
    private String updateCmd;
    private String updateWhere;
    private String deleteCmd;
    private String deleteWhere;
    private String insertCmd;
    private int[] keyCols;
    private Object[] params;
    private CachedRowSetReader reader;
    private ResultSetMetaData callerMd;
    private int callerColumnCount;
    private CachedRowSetImpl crsResolve;
    private ArrayList<Integer> status;
    private int iChangedValsInDbAndCRS;
    private int iChangedValsinDbOnly;
    private JdbcRowSetResourceBundle resBundle;
    static final long serialVersionUID = -8506030970299413976L;
    
    public CachedRowSetWriter() {
        try {
            this.resBundle = JdbcRowSetResourceBundle.getJdbcRowSetResourceBundle();
        }
        catch (IOException cause) {
            throw new RuntimeException(cause);
        }
    }
    
    @Override
    public boolean writeData(RowSetInternal rowSetInternal) throws SQLException {
        long lng = 0L;
        PreparedStatement prepareStatement = null;
        this.iChangedValsInDbAndCRS = 0;
        this.iChangedValsinDbOnly = 0;
        CachedRowSetImpl cachedRowSet = (CachedRowSetImpl)rowSetInternal;
        this.crsResolve = new CachedRowSetImpl();
        this.con = this.reader.connect(rowSetInternal);
        if (this.con == null) {
            throw new SQLException(this.resBundle.handleGetObject("crswriter.connect").toString());
        }
        this.initSQLStatements(cachedRowSet);
        RowSetMetaDataImpl rowSetMetaDataImpl = (RowSetMetaDataImpl)cachedRowSet.getMetaData();
        RowSetMetaDataImpl metaData = new RowSetMetaDataImpl();
        int columnCount = rowSetMetaDataImpl.getColumnCount();
        (this.status = new ArrayList(cachedRowSet.size() + 1)).add(0, null);
        metaData.setColumnCount(columnCount);
        for (int i = 1; i <= columnCount; ++i) {
            metaData.setColumnType(i, rowSetMetaDataImpl.getColumnType(i));
            metaData.setColumnName(i, rowSetMetaDataImpl.getColumnName(i));
            metaData.setNullable(i, 2);
        }
        this.crsResolve.setMetaData(metaData);
        if (this.callerColumnCount < 1) {
            if (this.reader.getCloseConnection()) {
                this.con.close();
            }
            return true;
        }
        else {
            int showDeleted = cachedRowSet.getShowDeleted();
            cachedRowSet.setShowDeleted(true);
            cachedRowSet.beforeFirst();
            for (int index = 1; cachedRowSet.next(); ++index) {
                if (cachedRowSet.rowDeleted()) {
                    if (this.deleteOriginalRow(cachedRowSet, this.crsResolve)) {
                        this.status.add(index, Integer.valueOf(1));
                        ++lng;
                    }
                    else {
                        this.status.add(index, Integer.valueOf(3));
                    }
                }
                else if (cachedRowSet.rowInserted()) {
                    prepareStatement = this.con.prepareStatement(this.insertCmd);
                    if (this.insertNewRow(cachedRowSet, prepareStatement, this.crsResolve)) {
                        this.status.add(index, Integer.valueOf(2));
                        ++lng;
                    }
                    else {
                        this.status.add(index, Integer.valueOf(3));
                    }
                }
                else if (cachedRowSet.rowUpdated()) {
                    if (this.updateOriginalRow(cachedRowSet)) {
                        this.status.add(index, Integer.valueOf(0));
                        ++lng;
                    }
                    else {
                        this.status.add(index, Integer.valueOf(3));
                    }
                }
                else {
                    cachedRowSet.getMetaData().getColumnCount();
                    this.status.add(index, Integer.valueOf(3));
                    this.crsResolve.moveToInsertRow();
                    for (int j = 0; j < columnCount; ++j) {
                        this.crsResolve.updateNull(j + 1);
                    }
                    this.crsResolve.insertRow();
                    this.crsResolve.moveToCurrentRow();
                }
            }
            if (prepareStatement != null) {
                prepareStatement.close();
            }
            cachedRowSet.setShowDeleted((boolean)showDeleted);
            cachedRowSet.beforeFirst();
            this.crsResolve.beforeFirst();
            if (lng != 0L) {
                SyncProviderException o = new SyncProviderException(lng + " " + this.resBundle.handleGetObject("crswriter.conflictsno").toString());
                SyncResolverImpl syncResolverImpl = (SyncResolverImpl)o.getSyncResolver();
                syncResolverImpl.setCachedRowSet(cachedRowSet);
                syncResolverImpl.setCachedRowSetResolver(this.crsResolve);
                syncResolverImpl.setStatus(this.status);
                syncResolverImpl.setCachedRowSetWriter(this);
                throw o;
            }
            return true;
        }
    }
    
    private boolean updateOriginalRow(CachedRowSet set) throws SQLException {
        int n = 0;
        ResultSet originalRow = set.getOriginalRow();
        originalRow.next();
        try {
            this.updateWhere = this.buildWhereClause(this.updateWhere, originalRow);
            int index = this.selectCmd.toLowerCase().indexOf("where");
            if (index != -1) {
                this.selectCmd = this.selectCmd.substring(0, index);
            }
            PreparedStatement prepareStatement = this.con.prepareStatement(this.selectCmd + this.updateWhere, 1005, 1007);
            for (int i = 0; i < this.keyCols.length; ++i) {
                if (this.params[i] != null) {
                    prepareStatement.setObject(++n, this.params[i]);
                }
            }
            try {
                prepareStatement.setMaxRows(set.getMaxRows());
                prepareStatement.setMaxFieldSize(set.getMaxFieldSize());
                prepareStatement.setEscapeProcessing(set.getEscapeProcessing());
                prepareStatement.setQueryTimeout(set.getQueryTimeout());
            }
            catch (Exception ex) {}
            ResultSet executeQuery = prepareStatement.executeQuery();
            executeQuery.getMetaData();
            if (executeQuery.next()) {
                if (executeQuery.next()) {
                    return true;
                }
                executeQuery.first();
                int n2 = 0;
                Vector<Integer> vector = new Vector();
                String str = this.updateCmd;
                Object x = null;
                int n3 = 1;
                int n4 = 1;
                this.crsResolve.moveToInsertRow();
                for (int j = 1; j <= this.callerColumnCount; ++j) {
                    Object object = originalRow.getObject(j);
                    Object object2 = set.getObject(j);
                    Object object3 = executeQuery.getObject(j);
                    Map<String, Class<?>> map = (set.getTypeMap() == null) ? this.con.getTypeMap() : set.getTypeMap();
                    if ((object3 instanceof Struct)) {
                        Struct struct = (Struct)object3;
                        Class<Object> clazz = (Class)map.get(struct.getSQLTypeName());
                        if (clazz != null) {
                            SQLData sqlData;
                            try {
                                sqlData = (SQLData)ReflectUtil.newInstance(clazz);
                            }
                            catch (Exception cause) {
                                throw new SQLException("Unable to Instantiate: ", cause);
                            }
                            sqlData.readSQL(new SQLInputImpl(struct.getAttributes(map), map), struct.getSQLTypeName());
                            object3 = sqlData;
                        }
                    }
                    else if ((object3 instanceof SQLData)) {
                        object3 = new SerialStruct((SQLData)object3, map);
                    }
                    else if ((object3 instanceof Blob)) {
                        object3 = new SerialBlob((Blob)object3);
                    }
                    else if ((object3 instanceof Clob)) {
                        object3 = new SerialClob((Clob)object3);
                    }
                    else if ((object3 instanceof Array)) {
                        object3 = new SerialArray((Array)object3, map);
                    }
                    boolean n5 = true;
                    if (object3 == null && object != null) {
                        ++this.iChangedValsinDbOnly;
                        n5 = false;
                        x = object3;
                    }
                    else if (object3 != null && !object3.equals(object)) {
                        ++this.iChangedValsinDbOnly;
                        n5 = false;
                        x = object3;
                    }
                    else if (object == null || object2 == null) {
                        if (n3 == 0 || n4 == 0) {
                            str += ", ";
                        }
                        String string2 = str + set.getMetaData().getColumnName(j);
                        vector.add(Integer.valueOf(j));
                        str = string2 + " = ? ";
                        n3 = 0;
                    }
                    else if (object.equals(object2)) {
                        ++n2;
                    }
                    else if (!object.equals(object2) && set.columnUpdated(j)) {
                        if (object3.equals(object)) {
                            if (n4 == 0 || n3 == 0) {
                                str += ", ";
                            }
                            String string = str + set.getMetaData().getColumnName(j);
                            vector.add(Integer.valueOf(j));
                            str = string + " = ? ";
                            n4 = 0;
                        }
                        else {
                            n5 = false;
                            x = object3;
                            ++this.iChangedValsInDbAndCRS;
                        }
                    }
                    if (n5 == 0) {
                        this.crsResolve.updateObject(j, x);
                    }
                    else {
                        this.crsResolve.updateNull(j);
                    }
                }
                executeQuery.close();
                prepareStatement.close();
                this.crsResolve.insertRow();
                this.crsResolve.moveToCurrentRow();
                if ((n3 == 0 && vector.size() == 0) || n2 == this.callerColumnCount) {
                    return false;
                }
                if (this.iChangedValsInDbAndCRS != 0 || this.iChangedValsinDbOnly != 0) {
                    return true;
                }
                PreparedStatement prepareStatement2 = this.con.prepareStatement(str + this.updateWhere);
                int k;
                for (k = 0; k < vector.size(); ++k) {
                    Object object4 = set.getObject((int)vector.get(k));
                    if (object4 != null) {
                        prepareStatement2.setObject(k + 1, object4);
                    }
                    else {
                        prepareStatement2.setNull(k + 1, set.getMetaData().getColumnType(k + 1));
                    }
                }
                for (int l = 0; l < this.keyCols.length; ++l) {
                    if (this.params[l] != null) {
                        prepareStatement2.setObject(++k, this.params[l]);
                    }
                }
                prepareStatement2.executeUpdate();
                return false;
            }
            else {
                return true;
            }
        }
        catch (SQLException ex2) {
            ex2.printStackTrace();
            this.crsResolve.moveToInsertRow();
            for (int columnIndex = 1; columnIndex <= this.callerColumnCount; ++columnIndex) {
                this.crsResolve.updateNull(columnIndex);
            }
            this.crsResolve.insertRow();
            this.crsResolve.moveToCurrentRow();
            return true;
        }
    }
    
    private boolean insertNewRow(CachedRowSet set, PreparedStatement preparedStatement, CachedRowSetImpl cachedRowSetImpl) throws SQLException {
        boolean n = false;
        try (final PreparedStatement prepareStatement = this.con.prepareStatement(this.selectCmd, 1005, 1007);
             final ResultSet executeQuery = prepareStatement.executeQuery();
             final ResultSet primaryKeys = this.con.getMetaData().getPrimaryKeys(null, null, set.getTableName())) {
            ResultSetMetaData metaData = set.getMetaData();
            int columnCount = metaData.getColumnCount();
            String[] array = new String[columnCount];
            int n2 = 0;
            while (primaryKeys.next()) {
                array[n2] = primaryKeys.getString("COLUMN_NAME");
                ++n2;
            }
            if (executeQuery.next()) {
                for (String anObject : array) {
                    if (this.isPKNameValid(anObject, metaData)) {
                        Object object = set.getObject(anObject);
                        if (object == null) {
                            break;
                        }
                        String string = executeQuery.getObject(anObject).toString();
                        if (object.toString().equals(string)) {
                            n = true;
                            this.crsResolve.moveToInsertRow();
                            for (int k = 1; k <= columnCount; ++k) {
                                if (executeQuery.getMetaData().getColumnName(k).equals(anObject)) {
                                    this.crsResolve.updateObject(k, string);
                                }
                                else {
                                    this.crsResolve.updateNull(k);
                                }
                            }
                            this.crsResolve.insertRow();
                            this.crsResolve.moveToCurrentRow();
                        }
                    }
                }
            }
            if (!(n == 0)) {
                return n;
            }
            else {
                try {
                    for (int l = 1; l <= columnCount; ++l) {
                        Object object2 = set.getObject(l);
                        if (object2 != null) {
                            preparedStatement.setObject(l, object2);
                        }
                        else {
                            preparedStatement.setNull(l, set.getMetaData().getColumnType(l));
                        }
                    }
                    preparedStatement.executeUpdate();
                    return false;
                }
                catch (SQLException ex) {
                    this.crsResolve.moveToInsertRow();
                    for (int i = 1; i <= columnCount; ++i) {
                        this.crsResolve.updateNull(i);
                    }
                    this.crsResolve.insertRow();
                    this.crsResolve.moveToCurrentRow();
                    return 1;
                }
            }
        }
    }
    
    private boolean deleteOriginalRow(CachedRowSet set, CachedRowSetImpl cachedRowSetImpl) throws SQLException {
        int n = 0;
        ResultSet originalRow = set.getOriginalRow();
        originalRow.next();
        this.deleteWhere = this.buildWhereClause(this.deleteWhere, originalRow);
        PreparedStatement prepareStatement = this.con.prepareStatement(this.selectCmd + this.deleteWhere, 1005, 1007);
        for (int i = 0; i < this.keyCols.length; ++i) {
            if (this.params[i] != null) {
                prepareStatement.setObject(++n, this.params[i]);
            }
        }
        try {
            prepareStatement.setMaxRows(set.getMaxRows());
            prepareStatement.setMaxFieldSize(set.getMaxFieldSize());
            prepareStatement.setEscapeProcessing(set.getEscapeProcessing());
            prepareStatement.setQueryTimeout(set.getQueryTimeout());
        }
        catch (Exception ex) {}
        ResultSet executeQuery = prepareStatement.executeQuery();
        if (executeQuery.next()) {
            if (executeQuery.next()) {
                return true;
            }
            executeQuery.first();
            boolean n2 = false;
            cachedRowSetImpl.moveToInsertRow();
            for (int j = 1; j <= set.getMetaData().getColumnCount(); ++j) {
                Object object = originalRow.getObject(j);
                Object object2 = executeQuery.getObject(j);
                if (object != null && object2 != null) {
                    if (!object.toString().equals(object2.toString())) {
                        n2 = true;
                        cachedRowSetImpl.updateObject(j, originalRow.getObject(j));
                    }
                }
                else {
                    cachedRowSetImpl.updateNull(j);
                }
            }
            cachedRowSetImpl.insertRow();
            cachedRowSetImpl.moveToCurrentRow();
            if (!(n2 == 0)) {
                return true;
            }
            PreparedStatement prepareStatement2 = this.con.prepareStatement(this.deleteCmd + this.deleteWhere);
            int n3 = 0;
            for (int k = 0; k < this.keyCols.length; ++k) {
                if (this.params[k] != null) {
                    prepareStatement2.setObject(++n3, this.params[k]);
                }
            }
            if (prepareStatement2.executeUpdate() != 1) {
                return true;
            }
            prepareStatement2.close();
            return false;
        }
        else {
            return true;
        }
    }
    
    public void setReader(CachedRowSetReader reader) throws SQLException {
        this.reader = reader;
    }
    
    public CachedRowSetReader getReader() throws SQLException {
        return this.reader;
    }
    
    private void initSQLStatements(CachedRowSet set) throws SQLException {
        this.callerMd = set.getMetaData();
        this.callerColumnCount = this.callerMd.getColumnCount();
        if (this.callerColumnCount < 1) {
            return;
        }
        String s = set.getTableName();
        if (s == null) {
            s = this.callerMd.getTableName(1);
            if (s == null || s.length() == 0) {
                throw new SQLException(this.resBundle.handleGetObject("crswriter.tname").toString());
            }
        }
        String catalogName = this.callerMd.getCatalogName(1);
        String schemaName = this.callerMd.getSchemaName(1);
        DatabaseMetaData metaData = this.con.getMetaData();
        this.selectCmd = "SELECT ";
        for (int i = 1; i <= this.callerColumnCount; ++i) {
            this.selectCmd += this.callerMd.getColumnName(i);
            if (i < this.callerMd.getColumnCount()) {
                this.selectCmd += ", ";
            }
            else {
                this.selectCmd += " ";
            }
        }
        this.selectCmd = this.selectCmd + "FROM " + this.buildTableName(metaData, catalogName, schemaName, s);
        this.updateCmd = "UPDATE " + this.buildTableName(metaData, catalogName, schemaName, s);
        int index = this.updateCmd.toLowerCase().indexOf("where");
        if (index != -1) {
            this.updateCmd = this.updateCmd.substring(0, index);
        }
        this.updateCmd += "SET ";
        this.insertCmd = "INSERT INTO " + this.buildTableName(metaData, catalogName, schemaName, s);
        this.insertCmd += "(";
        for (int j = 1; j <= this.callerColumnCount; ++j) {
            this.insertCmd += this.callerMd.getColumnName(j);
            if (j < this.callerMd.getColumnCount()) {
                this.insertCmd += ", ";
            }
            else {
                this.insertCmd += ") VALUES (";
            }
        }
        for (int k = 1; k <= this.callerColumnCount; ++k) {
            this.insertCmd += "?";
            if (k < this.callerColumnCount) {
                this.insertCmd += ", ";
            }
            else {
                this.insertCmd += ")";
            }
        }
        this.deleteCmd = "DELETE FROM " + this.buildTableName(metaData, catalogName, schemaName, s);
        this.buildKeyDesc(set);
        return;
    }
    
    private String buildTableName(DatabaseMetaData databaseMetaData, String trim, String trim2, String trim3) throws SQLException {
        String str = "";
        trim = trim.trim();
        trim2 = trim2.trim();
        trim3 = trim3.trim();
        String s;
        if (databaseMetaData.isCatalogAtStart()) {
            if (trim != null && trim.length() > 0) {
                str = str + trim + databaseMetaData.getCatalogSeparator();
            }
            if (trim2 != null && trim2.length() > 0) {
                str = str + trim2 + ".";
            }
            s = str + trim3;
        }
        else {
            if (trim2 != null && trim2.length() > 0) {
                str = str + trim2 + ".";
            }
            s = str + trim3;
            if (trim != null && trim.length() > 0) {
                s = s + databaseMetaData.getCatalogSeparator() + trim;
            }
        }
        return s + " ";
    }
    
    private void buildKeyDesc(CachedRowSet set) throws SQLException {
        this.keyCols = set.getKeyColumns();
        ResultSetMetaData metaData = set.getMetaData();
        if (this.keyCols == null || this.keyCols.length == 0) {
            ArrayList<Integer> list = new ArrayList();
            for (int i = 0; i < this.callerColumnCount; ++i) {
                if (metaData.getColumnType(i + 1) != 2005 && metaData.getColumnType(i + 1) != 2002 && metaData.getColumnType(i + 1) != 2009 && metaData.getColumnType(i + 1) != 2004 && metaData.getColumnType(i + 1) != 2003 && metaData.getColumnType(i + 1) != 1111) {
                    list.add(Integer.valueOf(i + 1));
                }
            }
            this.keyCols = new int[list.size()];
            for (int j = 0; j < list.size(); ++j) {
                this.keyCols[j] = (int)list.get(j);
            }
        }
        this.params = new Object[this.keyCols.length];
    }
    
    private String buildWhereClause(String s, ResultSet set) throws SQLException {
        s = "WHERE ";
        for (int i = 0; i < this.keyCols.length; ++i) {
            if (i > 0) {
                s += "AND ";
            }
            s += this.callerMd.getColumnName(this.keyCols[i]);
            this.params[i] = set.getObject(this.keyCols[i]);
            if (set.wasNull()) {
                s += " IS NULL ";
            }
            else {
                s += " = ? ";
            }
        }
        return s;
    }
    
    void updateResolvedConflictToDB(CachedRowSet set, Connection connection) throws SQLException {
        String s = "WHERE ";
        int columnCount = set.getMetaData().getColumnCount();
        int[] keyColumns = set.getKeyColumns();
        String str = "";
        this.buildWhereClause(s, set);
        if (keyColumns == null || keyColumns.length == 0) {
            keyColumns = new int[columnCount];
            for (int i = 0; i < keyColumns.length; keyColumns[i] = ++i) {}
        }
        Object[] array = new Object[keyColumns.length];
        String string = "UPDATE " + this.buildTableName(connection.getMetaData(), set.getMetaData().getCatalogName(1), set.getMetaData().getSchemaName(1), set.getTableName()) + "SET ";
        int n = 1;
        for (int j = 1; j <= columnCount; ++j) {
            if (set.columnUpdated(j)) {
                if (n == 0) {
                    str += ", ";
                }
                str = str + set.getMetaData().getColumnName(j) + " = ? ";
                n = 0;
            }
        }
        String string2 = string + str;
        String str2 = "WHERE ";
        for (int k = 0; k < keyColumns.length; ++k) {
            if (k > 0) {
                str2 += "AND ";
            }
            String string3 = str2 + set.getMetaData().getColumnName(keyColumns[k]);
            array[k] = set.getObject(keyColumns[k]);
            if (set.wasNull()) {
                str2 = string3 + " IS NULL ";
            }
            else {
                str2 = string3 + " = ? ";
            }
        }
        PreparedStatement prepareStatement = connection.prepareStatement(string2 + str2);
        int n2 = 0;
        for (int l = 0; l < columnCount; ++l) {
            if (set.columnUpdated(l + 1)) {
                Object object = set.getObject(l + 1);
                if (object != null) {
                    prepareStatement.setObject(++n2, object);
                }
                else {
                    prepareStatement.setNull(l + 1, set.getMetaData().getColumnType(l + 1));
                }
            }
        }
        for (int n3 = 0; n3 < keyColumns.length; ++n3) {
            if (array[n3] != null) {
                prepareStatement.setObject(++n2, array[n3]);
            }
        }
        prepareStatement.executeUpdate();
    }
    
    @Override
    public void commit() throws SQLException {
        this.con.commit();
        if (this.reader.getCloseConnection()) {
            this.con.close();
        }
    }
    
    public void commit(CachedRowSetImpl cachedRowSetImpl, boolean b) throws SQLException {
        this.con.commit();
        if (b && cachedRowSetImpl.getCommand() != null) {
            cachedRowSetImpl.execute(this.con);
        }
        if (this.reader.getCloseConnection()) {
            this.con.close();
        }
    }
    
    @Override
    public void rollback() throws SQLException {
        this.con.rollback();
        if (this.reader.getCloseConnection()) {
            this.con.close();
        }
    }
    
    @Override
    public void rollback(Savepoint savepoint) throws SQLException {
        this.con.rollback(savepoint);
        if (this.reader.getCloseConnection()) {
            this.con.close();
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        try {
            this.resBundle = JdbcRowSetResourceBundle.getJdbcRowSetResourceBundle();
        }
        catch (IOException cause) {
            throw new RuntimeException(cause);
        }
    }
    
    private boolean isPKNameValid(String anotherString, ResultSetMetaData resultSetMetaData) throws SQLException {
        boolean n = false;
        int columnCount = resultSetMetaData.getColumnCount();
        int i = 1;
        while (i <= columnCount) {
            if (resultSetMetaData.getColumnClassName(i).equalsIgnoreCase(anotherString)) {
                n = true;
                break;
            }
            ++i;
        }
        return n;
    }
}
